#include <stdio.h>
#include <time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define SIZEX 80
#define SIZEY 15

Display *dpy;            
Window root, win;
int scrn;
struct tm *tmval;
char color[10]="gray";
char color_bg[10]="darkblue";

char now[15];
GC gc,gc2,gc3;
Font font;   
XImage *bg;

char *window_name={"BadClock"}; 
int x,y,width,height;
char draw_border, transparent, fix;

void drawBadClock(){
	time_t curtime;

	curtime = time(NULL);
	tmval = localtime(&curtime);	

	sprintf(now,"%s%d:%s%d:%s%d",
		(tmval->tm_hour>9?"":"0"),tmval->tm_hour,
		(tmval->tm_min>9?"":"0"),tmval->tm_min,
		(tmval->tm_sec>9?"":"0"),tmval->tm_sec
	);	

	// If transparent ... draw the bg
	if(transparent)
		XPutImage(dpy, win, gc, bg, 0,0,0,0,width,height);	
	else	
		XClearWindow(dpy, win);

	XDrawString(dpy, win, gc, 2, 12, now, strlen(now));

	// Draw borders
    if(draw_border) {
        XDrawLine(dpy, win, gc2, 0, 0, width, 0);
        XDrawLine(dpy, win, gc2, 0, 0, 0, height);      
        XDrawLine(dpy, win, gc3, 1, height-1, width, height-1);
    	XDrawLine(dpy, win, gc3, width-1, 1, width-1, height);  
	}
}

void GetBackground(){
	// Get the image under our window
	bg = XGetImage(dpy, root, x, y, width, height, -1, ZPixmap); 
}

unsigned long GetColor(char *color)
{  
  Colormap cmap;
  XColor c0, c1;
  cmap = DefaultColormap(dpy, 0);
  XAllocNamedColor(dpy, cmap, color, &c1, &c0);
  return(c1.pixel);
} 

unsigned long MakeBorderColor(char *color, char fgbg){
        Colormap cmap;
        char bordercolor[8] = {0,};
        XColor c0, c1;
        unsigned short r,g,b;

        cmap = DefaultColormap(dpy, 0);
        XAllocNamedColor(dpy, cmap, color, &c1, &c0);
        r = c1.red;
        g = c1.green;
        b = c1.blue;
        
        // If leftupper
        if(fgbg == 1){
                if(r+20000 <= 65535 && g+20000 <= 56635 && b+20000 <= 56635){
                        r+=20000; g+=20000; b+=20000;
                }else{
                        // Just white
                        r = g = b = 56635;
                }
        }
        // If bottomright
        else{
                if(r-40000 >= 0 && g-40000 >= 0 && b-40000 >= 0){
                        r-=40000; g-=40000; b-=40000;
                }else{
                        // Just black
                        r = g = b = 0;
                }
        }

        r = r/256;
        g = g/256;
        b = b/256;
        
        sprintf(bordercolor, "#%lX%lX%lX",r,g,b);
        
        XAllocNamedColor(dpy, cmap, bordercolor, &c1, &c0);
        return c1.pixel;
}

void win_create(){
	XSetWindowAttributes attr;
    XSizeHints szhints;
    unsigned long mask;
    
	if(fix){ 
		attr.border_pixel = BlackPixel(dpy,scrn);
   		attr.background_pixel = GetColor(color_bg);
    	attr.override_redirect = True;                                                                 
    	attr.event_mask = NULL;
    	mask = CWBackPixel|CWBorderPixel|CWOverrideRedirect;
		win = XCreateWindow(dpy,root,x,y,width,height,0,DefaultDepth(dpy,scrn),InputOutput,CopyFromParent,mask,&attr);

		szhints.flags = PPosition | PSize;
   		szhints.x = x;
   	 	szhints.y = y;
    	szhints.width = width;
    	szhints.height = height;
        
    	XSetNormalHints(dpy,win,&szhints);
	}else{
		win = XCreateSimpleWindow(dpy, root, x, y, width, height, 0, 0, GetColor(color_bg));
	}
}

int main(int argc, char **argv){	
	int i;
	char myFont[70];
	width = SIZEX;
	height = SIZEY;
	x = 1;
	y = 1;

	draw_border = 1;
	transparent = fix = 0; // Transparency
	sprintf(myFont, "-misc-fixed-*-*-*-*-18-*-*-*-*-*-*-*");

	/* Parse argv */
    for (i=1; i < argc; i++) {
       if (!strcmp(argv[i],"-g") || !strcmp(argv[i],"--geometry")){
          XParseGeometry(argv[i+1],&x,&y,&width,&height);
       }
	   else if (!strcmp(argv[i],"-fg")){
		  sprintf(color,"%s",argv[i+1]);
	   }
	   else if (!strcmp(argv[i],"-bg")){
		  sprintf(color_bg,"%s",argv[i+1]);
	   }		
	   else if (!strcmp(argv[i],"-f")){
          sprintf(myFont,"%s",argv[i+1]);
       }
   	   else if (!strcmp(argv[i],"-b")){
          draw_border = 1;
       }
   	   else if (!strcmp(argv[i],"+b")){
          draw_border = 0;
       }
	   else if (!strcmp(argv[i],"-tr")){
          transparent = 1;
       }
	   else if (!strcmp(argv[i],"-x")){
		  fix = 1;
	   }
  	   else if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help")) {
          printf("usage: BadClock [--geometry -x -fg -bg -tr -f[ont]]\n");
          exit(0);
       }
    }

	dpy = XOpenDisplay(NULL);
  	scrn = DefaultScreen(dpy);
	root = RootWindow(dpy,scrn);	
	// Create window
	win_create();

	XStoreName(dpy,win,window_name);
	gc = XCreateGC(dpy,win,0,NULL);

	if(draw_border){
		gc2 = XCreateGC(dpy,win,0,NULL);
    	gc3 = XCreateGC(dpy,win,0,NULL);
	}
	XSetForeground(dpy,gc,GetColor(color));
	XSetBackground(dpy,gc,GetColor(color_bg));
    if(draw_border) {
        XSetForeground(dpy,gc2,MakeBorderColor(color_bg,1));
        XSetForeground(dpy,gc3,MakeBorderColor(color_bg,2));
    }

	font = XLoadFont(dpy, myFont);
  	XSetFont(dpy, gc, font);

	// Get bg image if transparent
	if(transparent)
		GetBackground();

	XMapWindow(dpy,win);

	while(1){ 
   		drawBadClock();
		XFlush(dpy);   
  		sleep(1);   
	}
	return 0;
}
